<?php

namespace EmailTunnel;

class ConnectionFinder {
    /**
     * Fetch a singular post by title and author ID.
     *
     * @param string $postTitle
     * @param int $postAuthorID
     * @param string $postType
     * @return mixed
     * @throws \Exception
     */
    public static function fetchSingularPostByTitleAndAuthor($postTitle, $postAuthorID, $postType = 'post') {
        $args = array(
            'post_type'      => $postType,
            'post_status'    => array('publish', 'draft'),
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'title'          => $postTitle,
            'post_author'    => $postAuthorID
        );

        $query = new \WP_Query($args);
        $post_ids = $query->posts;

        if (count($post_ids) === 1) {
            return get_post($post_ids[0]);
        } elseif (empty($post_ids)) {
            return false;
        } else {
            throw new \Exception("ERROR: more than one object meets the search criteria.");
        }
    }

    /**
     * Fetch a singular etm-connection by title, author ID, and remote-site-url.
     *
     * @param string $postTitle
     * @param int $postAuthorID
     * @return mixed
     * @throws \Exception
     */
    public static function fetchSingularETM_Connection($postTitle, $postAuthorID, $remoteSiteURL, $postType = 'etm-connection') {
        $args = array(
            'post_type'      => $postType,
            'post_status'    => array('publish', 'draft'),
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'title'          => $postTitle,
            'author'         => $postAuthorID
        );

        $query = new \WP_Query($args);
        $post_ids = $query->posts;

        // Filter post IDs by the matching 'remote-site-url' meta data
        foreach ($post_ids as $key => $id) {
            $remote_url_meta = get_post_meta($id, 'remote-site-url', true);
            if ($remote_url_meta !== $remoteSiteURL) {
                unset($post_ids[$key]); // Remove the post ID if it doesn't match the remote URL
            }
        }

        // Reset array keys after filtering
        $post_ids = array_values($post_ids);

        if (count($post_ids) === 1) {
            return get_post($post_ids[0]); // Return the post object of the single remaining post ID
        } elseif (empty($post_ids)) {
            return false; // Return false if no posts meet the criteria
        } else {
            throw new \Exception("ERROR: more than one object meets the search criteria.");
        }
    }

    public static function fetchValidatedNodeConnection() {
        global $wpdb;

        // Define the custom post type and title
        $post_type = 'etm-connection';
        $post_title = 'THIS IS A VALIDATED REMOTE NODE';

        // Query to fetch posts with the given title and post type
        $query = $wpdb->prepare("
        SELECT ID 
        FROM {$wpdb->posts} 
        WHERE post_type = %s 
        AND post_title = %s
        AND post_status = 'draft'
    ", $post_type, $post_title);

        $results = $wpdb->get_results($query);

        // Handle more than one or no results as before
        if (count($results) > 1) {
            self::deleteAllConnections("etm-connection");
            return false;
        } elseif (count($results) == 1) {
            $post_id = $results[0]->ID;
            // Check for required meta data
            $required_meta_keys = ['nonce', 'mothership-nonce', 'etm-nonce'];
            foreach ($required_meta_keys as $meta_key) {
                if (!get_post_meta($post_id, $meta_key, true)) {
                    // If any meta key is missing, delete the post and return false
                    self::deleteAllConnections("etm-connection");
                    return false;
                }
            }
            // If all meta data are present, return the post
            return get_post($post_id);
        } else {
            return false;
        }
    }


    public static function get_post_type_by_title( $page_title, $output = OBJECT, $post_type = 'page' ) {
        //_deprecated_function( __FUNCTION__, '6.2.0', 'WP_Query' );
        global $wpdb;

        if ( is_array( $post_type ) ) {
            $post_type           = esc_sql( $post_type );
            $post_type_in_string = "'" . implode( "','", $post_type ) . "'";
            $sql                 = $wpdb->prepare(
                "SELECT ID
			FROM $wpdb->posts
			WHERE post_title = %s
			AND post_type IN ($post_type_in_string)",
                $page_title
            );
        } else {
            $sql = $wpdb->prepare(
                "SELECT ID
			FROM $wpdb->posts
			WHERE post_title = %s
			AND post_type = %s",
                $page_title,
                $post_type
            );
        }

        $page = $wpdb->get_var( $sql );

        if ( $page ) {
            return get_post( $page, $output );
        }

        return null;
    }

    /**
     * Deletes all posts of a specified custom post type, forcing them to be permanently deleted.
     *
     * @param string $customPostType The name of the custom post type to delete.
     * @return bool True upon success, false if there is an error.
     */
    public static function deleteAllConnections($customPostType)
    {
        $args = array(
            'post_type' => $customPostType,
            'posts_per_page' => -1,
            'post_status' => 'any'
        );
        $posts = get_posts($args);
        if ($posts) {
            foreach ($posts as $post) {
                \wp_delete_post($post->ID, true);
            }
        }
    }

    /*
    public static function xget_post_type_by_title($page_title, $output = OBJECT, $post_type = 'page')
    {
        global $wpdb;

        if (is_array($post_type)) {
            $post_type = esc_sql($post_type);
            $post_type_in_string = "'" . implode("','", $post_type) . "'";
            $sql = $wpdb->prepare(
                "
            SELECT ID
            FROM $wpdb->posts
            WHERE post_title = %s
            AND post_type IN ($post_type_in_string)
        ",
                $page_title
            );
        } else {
            $sql = $wpdb->prepare(
                "
            SELECT ID
            FROM $wpdb->posts
            WHERE post_title = %s
            AND post_type = %s
        ",
                $page_title,
                $post_type
            );
        }

        $page = $wpdb->get_var($sql);

        if ($page) {
            return get_post($page, $output);
        }

        return null;
    }
    */
}